import XLSX from 'xlsx'

export const blobType = {
  xls: 'application/vnd.ms-excel',
  xlsx: 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
  csv: 'text/csv',
  doc: 'application/msword',
  docx: 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
  pdf: 'application/pdf',
  ppt: 'application/vnd.ms-powerpoint',
  pptx: 'application/vnd.openxmlformats-officedocument.presentationml.presentation',
  png: 'image/png',
  gif: 'image/gif',
  jpeg: 'image/jpeg',
  jpg: 'image/jpeg',
  mp3: 'audio/mpeg',
  aac: 'audio/aac',
  html: 'text/html',
  css: 'text/css',
  js: 'text/javascript',
  json: 'application/json',
  abw: 'application/x-abiword',
  arc: 'application/x-freearc',
  avi: 'video/x-msvideo',
  azw: 'application/vnd.amazon.ebook',
  bin: 'application/octet-stream',
  bmp: 'image/bmp',
  bz: 'application/x-bzip',
  bz2: 'application/x-bzip2',
  csh: 'application/x-csh',
  eot: 'application/vnd.ms-fontobject',
  epub: 'application/epub+zip',
  htm: 'text/html',
  ico: 'image/vnd.microsoft.icon',
  ics: 'text/calendar',
  jar: 'application/java-archive',
  jsonld: 'application/ld+json',
  mid: 'audio/midi audio/x-midi',
  midi: 'audio/midi audio/x-midi',
  mjs: 'text/javascript',
  mpeg: 'video/mpeg',
  mpkg: 'application/vnd.apple.installer+xml',
  odp: 'application/vnd.oasis.opendocument.presentation',
  ods: 'application/vnd.oasis.opendocument.spreadsheet',
  odt: 'application/vnd.oasis.opendocument.text',
  oga: 'audio/ogg',
  ogv: 'video/ogg',
  ogx: 'application/ogg',
  otf: 'font/otf',
  rar: 'application/x-rar-compressed',
  rtf: 'application/rtf',
  sh: 'application/x-sh',
  svg: 'image/svg+xml',
  swf: 'application/x-shockwave-flash',
  tar: 'application/x-tar',
  tif: 'image/tiff',
  tiff: 'image/tiff',
  ttf: 'font/ttf',
  txt: 'text/plain',
  vsd: 'application/vnd.visio',
  wav: 'audio/wav',
  weba: 'audio/webm',
  webm: 'video/webm',
  webp: 'image/webp',
  woff: 'font/woff',
  woff2: 'font/woff2',
  xhtml: 'application/xhtml+xml',
  xml: 'text/xml',
  xul: 'application/vnd.mozilla.xul+xml',
  zip: 'application/zip'
}

export const toBase64 = (str) => {
  // 使用 TextEncoder 将字符串编码为 UTF-8
  const utf8Bytes = new TextEncoder().encode(str)
  // 将字节数组转换为二进制字符串
  let binaryString = ''
  const len = utf8Bytes.byteLength
  for (let i = 0; i < len; i++) {
    binaryString += String.fromCharCode(utf8Bytes[i])
  }
  // 使用 btoa 对二进制字符串进行 base64 编码
  return btoa(binaryString)
}

export const handleReadFile = (file, type = false, callback) => {
  const fileReader = new FileReader()
  fileReader.onloadend = (e) => {
    if (type === 'xls') {
      const data = new Uint8Array(e.target.result)
      const workbook = XLSX.read(data, { type: 'array' })
      const newWorkbook = XLSX.utils.book_new()
      workbook.SheetNames.forEach(ele => {
        XLSX.utils.book_append_sheet(newWorkbook, workbook.Sheets[ele], ele)
      })
      const xlsxFile = XLSX.write(newWorkbook, { bookType: 'xlsx', type: 'array' })
      const fileArray = new Uint8Array(xlsxFile)
      const blob = new Blob([fileArray], { type: blobType.xlsx })
      const xlsxFileObject = new File([blob], 'output.xlsx', { type: blobType.xlsx })
      callback(xlsxFileObject)
    } else {
      callback(e.target.result)
    }
  }
  fileReader.readAsArrayBuffer(file)
}
